package com.imageresizer;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.ImageDecoder;
import android.graphics.drawable.Drawable;
import android.util.Log;
import android.webkit.MimeTypeMap;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bumptech.glide.Glide;
import com.bumptech.glide.gifdecoder.StandardGifDecoder;
import com.bumptech.glide.integration.webp.decoder.WebpDecoder;
import com.bumptech.glide.integration.webp.decoder.WebpDrawable;
import com.bumptech.glide.load.DataSource;
import com.bumptech.glide.load.Encoder;
import com.bumptech.glide.load.Options;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.load.engine.GlideException;
import com.bumptech.glide.load.resource.gif.GifDrawable;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.CustomTarget;
import com.bumptech.glide.request.target.Target;
import com.bumptech.glide.request.transition.Transition;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.n4no.webpencoder.codec.WebpBitmapEncoder;
import com.waynejo.androidndkgif.GifEncoder;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class ImageResizerModule extends ReactContextBaseJavaModule {
  public static String TAG = "ImageResizerModule";
  public static final String NAME = "ImageResizer";
  // Assuming you have a class variable/member
  private final ExecutorService executorService = Executors.newSingleThreadExecutor();
  ImageResizerModule(ReactApplicationContext context) {
    super(context);
  }

  @Override
  @NonNull
  public String getName() {
    return NAME;
  }


  // Example method
  // See https://reactnative.dev/docs/native-modules-android


  @Override
  public Map<String, Object> getConstants() {

    final Map<String, Object> constants = new HashMap<>();
    Map<String, String> scale = ScaleMode.asMap();
    Map<String, String> format = ImageFormatType.asMap();

    constants.put("Scale", scale);
    constants.put("Format", format);
    return constants;
  }

  @ReactMethod
  public void TestG(String uri, ReadableMap userOptions, Promise promise) {
    Context context = getReactApplicationContext();
    StaticOptions options = new StaticOptions(userOptions);
    executorService.submit(() -> {
      try {
        GifDrawable resource = Glide.with(context)
                .asGif()
                .load(uri)
                .submit()
                .get();


        AnimatedDecoder decoder = DecoderFactory.getDecoder(resource);


        File outputFile = new FileUtility().createOutputFile(options.outFormat);

//        MyEncoder encoder = EncoderFactory.createEncoder("gif", outputFile, 100, 100);
//        GifEncoder encoder =  new GifEncoder();
//        encoder.init(options.width, options.height, outputFile.getAbsolutePath());

        WebpBitmapEncoder encoder = new WebpBitmapEncoder(outputFile);
        encoder.setLoops(0);

        for(int i=0; i<decoder.getFrameCount(); i++){
          Bitmap ogBitmap = decoder.getNextFrame();
          Bitmap bitmap = BitmapUtility.resizeBitmap(ogBitmap, options );
          if(bitmap == null){
            promise.reject("broke");
          }
          encoder.setDuration(decoder.getNextDelay());
          encoder.writeFrame(bitmap,80);
          Log.d(TAG,"encoding i "+i);
          decoder.advance();
        }

        encoder.close();

        // Handle successful loading
        resolvePromise(promise);
      } catch (Exception e) {
        // Handle errors gracefully
        rejectPromise(promise, e);
      }
    });


  }
  @ReactMethod
  public void TestW(String uri, ReadableMap userOptions, Promise promise) {
    Context context = getReactApplicationContext();

    executorService.submit(() -> {
      try {
        WebpDrawable resource = Glide.with(context)
                .as(WebpDrawable.class)
                .load(uri)
                .submit()
                .get();


        AnimatedDecoder decoder = DecoderFactory.getDecoder(resource);

        Log.d(TAG,"Decoder "+decoder.getDecoderName());
        Log.d(TAG,"getFrameCount "+decoder.getFrameCount());
        Log.d(TAG,"getDelay "+decoder.getDelay(0));
        Log.d(TAG,"Height "+decoder.getNextFrame().getWidth());
        Log.d(TAG,"Width "+decoder.getNextFrame().getHeight());


        // Handle successful loading
        resolvePromise(promise);
      } catch (Exception e) {
        // Handle errors gracefully
        rejectPromise(promise, e);
      }
    });


  }
  private void resolvePromise(Promise promise) {
    // Create a success response object based on your promise library
    promise.resolve("resolved");
  }

  private void rejectPromise(Promise promise, Exception e) {
    // Create an error response object based on your promise library
    promise.reject(e);
  }
  @ReactMethod
  public void ResizeStaticImage(String uri, ReadableMap userOptions, Promise promise) {
    StaticOptions options = new StaticOptions(userOptions);

    File file = new File(uri);
    if (!file.exists()) {
      promise.reject("File does not exist");
    }

    // create bitmap
    Bitmap bitmap = BitmapFactory.decodeFile(uri);

    //  re-scale
    Bitmap newBitmap = BitmapUtility.resizeBitmap(bitmap, options);

    // create file
    try {
      File newFile = BitmapUtility.saveBitmapToFile(newBitmap, options.outFormat, options.quality);
      promise.resolve(newFile.getAbsolutePath());
    } catch (FileNotFoundException e) {
      promise.reject("Error in ResizeStaticImage:", e);
    }


  }

  @ReactMethod
  public void ResizeStaticImagesBatch(ReadableArray uris, ReadableMap userOptions, Promise promise) {
    StaticOptions options = new StaticOptions(userOptions);

    try {
      // Create an array to store the paths of resized images
      WritableArray resizedPaths = Arguments.createArray();

      for (int i = 0; i < uris.size(); i++) {
        String uri = uris.getString(i);

        File file = new File(uri);
        if (!file.exists()) {
          // Add an error message to the array
          resizedPaths.pushString("Error: File does not exist for URI: " + uri);
          continue;
        }

        // create bitmap
        Bitmap bitmap = BitmapFactory.decodeFile(uri);

        // re-scale
        Bitmap newBitmap = BitmapUtility.resizeBitmap(bitmap, options);

        // create file
        File newFile = BitmapUtility.saveBitmapToFile(newBitmap, options.outFormat, options.quality);
        resizedPaths.pushString(newFile.getAbsolutePath());
      }

      promise.resolve(resizedPaths);
    } catch (Exception e) {
      promise.reject("Error in ResizeStaticImagesBatch:", e);
    }
  }

  @ReactMethod
  public void ResizeAnimatedImage(String uri, ReadableMap userOptions, Promise promise) {

    try{
      StaticOptions options = new StaticOptions(userOptions);

      File file = new File(uri);
      if (!file.exists()) promise.reject("Error:ResizeAnimatedImage", "File does not exist");

      Context context = getReactApplicationContext();

      String extension = MimeTypeMap.getFileExtensionFromUrl(uri);
      if(extension != null){

        // on complete
        CodecCallback cb = new CodecCallback() {
          @Override
          public void onComplete(String filePath) {
            promise.resolve(filePath);
          }

          @Override
          public void onFailed(String filePath) {
            promise.reject("Error in codec",filePath);
          }
        };

        // Call as per Type GIF/WEBP
        if(extension.equalsIgnoreCase("gif")){
          GlideResizeGif(uri, cb, options);
        }else if(extension.equals("webp")){
          Log.d(TAG,"selected webp decoding. ");
          GlideResizeAwebp(uri, cb, options);
        }else{
          promise.reject("Error:ResizeAnimatedImage", "File extentions is not gif or webp");
        }
      }
    }catch (Exception e){
      Log.d(TAG,"Exception:ResizeAnimatedImage "+e.getMessage());
      promise.reject(TAG,e);
    }
  }


    private void GlideResizeGif(String uri, CodecCallback callback, StaticOptions options) {
    Context context = getReactApplicationContext();

    // 1. init Target type
    TargetGif gifTarget = new TargetGif(context, callback, options);


    // 2. call Glide
    Glide.with(context)
            .asGif()
            .load(uri)
            .skipMemoryCache(true)
            .diskCacheStrategy(DiskCacheStrategy.NONE)
            .listener(new RequestListener<GifDrawable>() {
              @Override
              public boolean onLoadFailed(@Nullable GlideException e, @Nullable Object model, @NonNull Target<GifDrawable> target, boolean isFirstResource) {
                if(e != null){
                  callback.onFailed(e.getMessage());
                }else{
                  callback.onFailed("Unknown error in Glide decoding GIF");
                }
                return false;
              }

              @Override
              public boolean onResourceReady(@NonNull GifDrawable resource, @NonNull Object model, Target<GifDrawable> target, @NonNull DataSource dataSource, boolean isFirstResource) {
                return false;
              }
            })
            .into(gifTarget);
  }

  private void GlideResizeAwebp(String uri, CodecCallback callback, StaticOptions options) {
    Context context = getReactApplicationContext();

    // 1. init Target type
    TargetWebp targetWebp = new TargetWebp(context, callback, options);

    // 2. call Glide
    Glide.with(context)
            .as(WebpDrawable.class)
            .load(uri)
            .skipMemoryCache(true)
            .diskCacheStrategy(DiskCacheStrategy.NONE)
            .listener(new RequestListener<>() {
              @Override
              public boolean onLoadFailed(@Nullable GlideException e, @Nullable Object model, @NonNull Target<WebpDrawable> target, boolean isFirstResource) {
                Log.d(TAG,"async webp decoding: error event heared. ");

                if (e != null) {
                  callback.onFailed(e.getMessage());
                } else {
                  callback.onFailed("Unknown error in Glide decoding GIF");
                }
                return false;
              }

              @Override
              public boolean onResourceReady(@NonNull WebpDrawable resource, @NonNull Object model, Target<WebpDrawable> target, @NonNull DataSource dataSource, boolean isFirstResource) {
                return false;
              }
            })
            .into(targetWebp);
    Log.d(TAG,"async webp decoding. ");

  }


}
